@@ -14,13 +14,20 @@
 """Common definitions to avoid circular imports
 """
 import threading
+
 import zope.interface
+
 from zope.security import interfaces
 
+
 thread_local = threading.local()
 
-class system_user(object):
-    zope.interface.classProvides(interfaces.IPrincipal)
+
+@zope.interface.implementer(interfaces.ISystemPrincipal)
+class SystemUser(object):
     id = u'zope.security.management.system_user'
     title = u'System'
     description = u''
+
+
+system_user = SystemUser()
