@@ -36,14 +36,16 @@ You can set the environment variable ``ZOPE_WATCH_CHECKERS`` before
 this module is imported to get additional security checker debugging
 output on the standard error.
 
-Setting ``ZOPE_WATCH_CHECKERS`` to 1 will display messages about unauthorized or
-forbidden attribute access.  Setting it to a larger number will also display
+Setting ``ZOPE_WATCH_CHECKERS`` to 1 will display messages about unauthorized
+or forbidden attribute access.  Setting it to a larger number will also display
 messages about granted attribute access.
 
 Note that the ``ZOPE_WATCH_CHECKERS`` mechanism may eventually be
 replaced with a more general security auditing mechanism.
 
-.. seealso:: :class:`CheckerLoggingMixin`, :class:`WatchingChecker`, :class:`WatchingCombinedChecker`
+.. seealso:: :class:`CheckerLoggingMixin`
+.. seealso:: :class:`WatchingChecker`
+.. seealso:: :class:`WatchingCombinedChecker`
 
 API
 ===
@@ -56,48 +58,49 @@ API
 .. autofunction:: selectChecker
 """
 import abc
+import datetime
+import decimal
 import os
 import sys
 import types
-import datetime
-import decimal
 import weakref
 
-from zope.i18nmessageid import Message
+import zope.interface.declarations
 import zope.interface.interface
 import zope.interface.interfaces
-import zope.interface.declarations
+from zope.i18nmessageid import Message
 from zope.interface import Interface
 from zope.interface import directlyProvides
 from zope.interface import implementer
 from zope.interface.interfaces import IDeclaration
 from zope.interface.interfaces import IInterface
 
+from zope.security._compat import CLASS_TYPES
+from zope.security._compat import PURE_PYTHON
+from zope.security._compat import PYTHON2
+from zope.security._compat import implementer_if_needed
+from zope.security._definitions import thread_local
+from zope.security.interfaces import ForbiddenAttribute
 from zope.security.interfaces import IChecker
 from zope.security.interfaces import INameBasedChecker
 from zope.security.interfaces import ISecurityProxyFactory
-from zope.security.interfaces import ForbiddenAttribute
 from zope.security.interfaces import Unauthorized
-from zope.security._definitions import thread_local
-from zope.security._compat import CLASS_TYPES
-from zope.security._compat import PYTHON2
-from zope.security._compat import PURE_PYTHON
-from zope.security._compat import implementer_if_needed
 from zope.security.proxy import Proxy
 from zope.security.proxy import getChecker
 
+
 try:
     from zope.exceptions import DuplicationError
-except ImportError: # pragma: no cover
+except ImportError:  # pragma: no cover
     class DuplicationError(Exception):
         """A duplicate registration was attempted"""
 
 WATCH_CHECKERS = 0
 
-if os.environ.get('ZOPE_WATCH_CHECKERS'):
+if os.environ.get('ZOPE_WATCH_CHECKERS'):  # pragma: no cover
     try:
         WATCH_CHECKERS = int(os.environ.get('ZOPE_WATCH_CHECKERS'))
-    except ValueError: # pragma: no cover
+    except ValueError:
         WATCH_CHECKERS = 1
 
 
@@ -129,12 +132,16 @@ def ProxyFactory(object, checker=None):
 
     return Proxy(object, checker)
 
+
 directlyProvides(ProxyFactory, ISecurityProxyFactory)
 
 # This import represents part of the API for the proxy module
-from . import proxy
+from . import proxy  # noqa: E402 module level import not at top
+
+
 proxy.ProxyFactory = ProxyFactory
 
+
 def canWrite(obj, name):
     """Check whether the interaction may write an attribute named name on obj.
 
@@ -169,6 +176,7 @@ def canWrite(obj, name):
     # should be passed through uncaught, as they indicate programmer error
     return True
 
+
 def canAccess(obj, name):
     """Check whether the interaction may access an attribute named name on obj.
 
@@ -187,6 +195,7 @@ def canAccess(obj, name):
     # indicates a programming or configuration error
     return True
 
+
 @implementer(INameBasedChecker)
 class CheckerPy(object):
     """
@@ -230,7 +239,6 @@ class CheckerPy(object):
         if self.set_permissions:
             return self.set_permissions.get(name)
 
-
     def check_setattr(self, object, name):
         'See IChecker'
         if self.set_permissions:
@@ -240,9 +248,9 @@ class CheckerPy(object):
 
         if permission is not None:
             if permission is CheckerPublic:
-                return # Public
+                return  # Public
             if thread_local.interaction.checkPermission(permission, object):
-                return # allowed
+                return  # allowed
             else:
                 __traceback_supplement__ = (TracebackSupplement, object)
                 raise Unauthorized(object, name, permission)
@@ -255,7 +263,7 @@ class CheckerPy(object):
         permission = self.get_permissions.get(name)
         if permission is not None:
             if permission is CheckerPublic:
-                return # Public
+                return  # Public
             if thread_local.interaction.checkPermission(permission, object):
                 return
             else:
@@ -268,7 +276,7 @@ class CheckerPy(object):
             __traceback_supplement__ = (TracebackSupplement, object)
             raise ForbiddenAttribute(name, object)
 
-    check_getattr = check # 'See IChecker'
+    check_getattr = check  # 'See IChecker'
 
     def proxy(self, value):
         'See IChecker'
@@ -281,7 +289,9 @@ class CheckerPy(object):
                 return value
 
         return Proxy(value, checker)
-Checker = CheckerPy # in case no C optimizations
+
+
+Checker = CheckerPy  # in case no C optimizations
 
 
 # Helper class for __traceback_supplement__
@@ -296,19 +306,19 @@ class TracebackSupplement(object):
             cls = self.obj.__class__
             if hasattr(cls, "__module__"):
                 s = "%s.%s" % (cls.__module__, cls.__name__)
-            else: # pragma: no cover XXX
+            else:  # pragma: no cover XXX
                 s = str(cls.__name__)
             result.append("   - class: " + s)
-        except: # pragma: no cover XXX
+        except:  # pragma: no cover # noqa: E722 do not use bare 'except'
             pass
         try:
             cls = type(self.obj)
             if hasattr(cls, "__module__"):
                 s = "%s.%s" % (cls.__module__, cls.__name__)
-            else: # pragma: no cover XXX
+            else:  # pragma: no cover XXX
                 s = str(cls.__name__)
             result.append("   - type: " + s)
-        except: # pragma: no cover XXX
+        except:  # pragma: no cover # noqa: E722 do not use bare 'except'
             pass
         return "\n".join(result)
 
@@ -322,7 +332,7 @@ class Global(object):
     """
 
     def __init__(self, name, module=None):
-        if module is None: # pragma: no cover XXX
+        if module is None:  # pragma: no cover XXX
             module = sys._getframe(1).f_locals['__name__']
 
         self.__name__ = name
@@ -344,7 +354,7 @@ CP_HACK_XXX = CheckerPublic
 # XXX: This means that we can't directly document it with
 # sphinx because issubclass() will fail.
 d = {}
-CheckerPublic = Proxy(CheckerPublic, Checker(d)) # XXX uses CheckerPy
+CheckerPublic = Proxy(CheckerPublic, Checker(d))  # XXX uses CheckerPy
 d['__reduce__'] = CheckerPublic
 d['__module__'] = CheckerPublic
 del d
@@ -353,6 +363,7 @@ del d
 # granted to __reduce__. We might want to bother to prevent this in
 # general and only allow it in this specific case.
 
+
 def NamesChecker(names=(), permission_id=CheckerPublic, **__kw__):
     """Return a checker that grants access to a set of names.
 
@@ -371,6 +382,7 @@ def NamesChecker(names=(), permission_id=CheckerPublic, **__kw__):
 
     return Checker(data)
 
+
 def InterfaceChecker(interface, permission_id=CheckerPublic, **__kw__):
     """
     Create a :func:`NamesChecker` for all the names defined in the *interface*
@@ -378,6 +390,7 @@ def InterfaceChecker(interface, permission_id=CheckerPublic, **__kw__):
     """
     return NamesChecker(interface.names(all=True), permission_id, **__kw__)
 
+
 def MultiChecker(specs):
     """
     Create a checker from a sequence of specifications
@@ -415,6 +428,7 @@ def MultiChecker(specs):
 
     return Checker(data)
 
+
 def selectCheckerPy(object):
     """Get a checker for the given object
 
@@ -428,13 +442,13 @@ def selectCheckerPy(object):
 
     # TODO: we really need formal proxy introspection
 
-    #if type(object) is Proxy:
+    # if type(object) is Proxy:
     #    # Is this already a security proxy?
     #    return None
 
     checker = _getChecker(type(object), _defaultChecker)
 
-    #checker = _getChecker(getattr(object, '__class__', type(object)),
+    # checker = _getChecker(getattr(object, '__class__', type(object)),
     #                      _defaultChecker)
 
     if checker is NoProxy:
@@ -447,12 +461,18 @@ def selectCheckerPy(object):
             return None
 
     return checker
-selectChecker = selectCheckerPy # in case no C optimizations
+
+
+selectChecker = selectCheckerPy  # in case no C optimizations
+
 
 def getCheckerForInstancesOf(class_):
     return _checkers.get(class_)
 
+
 DEFINABLE_TYPES = CLASS_TYPES + (types.ModuleType,)
+
+
 def defineChecker(type_, checker):
     """Define a checker for a given type of object
 
@@ -466,9 +486,11 @@ def defineChecker(type_, checker):
         raise DuplicationError(type_)
     _checkers[type_] = checker
 
+
 def undefineChecker(type_):
     del _checkers[type_]
 
+
 NoProxy = object()
 
 # _checkers is a mapping.
@@ -488,22 +510,25 @@ _available_by_default = []
 
 # Get optimized versions
 _c_available = not PURE_PYTHON
-if _c_available:
+if _c_available:  # pragma: no cover
     try:
         import zope.security._zope_security_checker
-    except (ImportError, AttributeError): # pragma: no cover PyPy / PURE_PYTHON
+    except (ImportError, AttributeError):
         _c_available = False
 
-if _c_available:
-    from zope.security._zope_security_checker import _checkers, selectChecker
-    from zope.security._zope_security_checker import NoProxy, Checker
-    from zope.security._zope_security_checker import _defaultChecker
+if _c_available:  # pragma: no cover
+    from zope.security._zope_security_checker import Checker
+    from zope.security._zope_security_checker import NoProxy
     from zope.security._zope_security_checker import _available_by_default
+    from zope.security._zope_security_checker import _checkers
+    from zope.security._zope_security_checker import _defaultChecker
+    from zope.security._zope_security_checker import selectChecker
     zope.interface.classImplements(Checker, INameBasedChecker)
 
 
 _getChecker = _checkers.get
 
+
 @implementer_if_needed(IChecker)
 class CombinedChecker(Checker):
     """A checker that combines two other checkers in a logical-or fashion.
@@ -515,7 +540,7 @@ class CombinedChecker(Checker):
     +====================+====================+=====================================+
     | ok                 | anything           | ok (checker 2 never called)         |
     +--------------------+--------------------+-------------------------------------+
-    | Unathorized        | ok                 | ok                                  |
+    | Unauthorized       | ok                 | ok                                  |
     +--------------------+--------------------+-------------------------------------+
     | Unauthorized       | Unauthorized       | Unauthorized                        |
     +--------------------+--------------------+-------------------------------------+
@@ -527,7 +552,7 @@ class CombinedChecker(Checker):
     +--------------------+--------------------+-------------------------------------+
     | ForbiddenAttribute | ForbiddenAttribute | ForbiddenAttribute                  |
     +--------------------+--------------------+-------------------------------------+
-    """
+    """  # noqa: E501 line too long
 
     def __init__(self, checker1, checker2):
         """Create a combined checker."""
@@ -544,7 +569,8 @@ class CombinedChecker(Checker):
         except ForbiddenAttribute:
             self._checker2.check(object, name)
         except Unauthorized as unauthorized_exception:
-            try: self._checker2.check(object, name)
+            try:
+                self._checker2.check(object, name)
             except ForbiddenAttribute:
                 raise unauthorized_exception
 
@@ -562,6 +588,7 @@ class CombinedChecker(Checker):
             except ForbiddenAttribute:
                 raise unauthorized_exception
 
+
 class CheckerLoggingMixin(object):
     """
     Debugging mixin for checkers.
@@ -649,6 +676,8 @@ class WatchingChecker(CheckerLoggingMixin, Checker):
     module is imported.
     """
     verbosity = WATCH_CHECKERS
+
+
 class WatchingCombinedChecker(CombinedChecker, WatchingChecker):
     """
     A checker that will perform verbose logging. This will be set
@@ -657,7 +686,8 @@ class WatchingCombinedChecker(CombinedChecker, WatchingChecker):
     """
     verbosity = WATCH_CHECKERS
 
-if WATCH_CHECKERS: # pragma: no cover
+
+if WATCH_CHECKERS:  # pragma: no cover
     # When we make these the default, we also need to be sure
     # to update the _defaultChecker's type (if it's not the C
     # extension) so that selectCheckerPy can properly recognize
@@ -669,9 +699,11 @@ if WATCH_CHECKERS: # pragma: no cover
     if not _c_available:
         _defaultChecker.__class__ = Checker
 
+
 def _instanceChecker(inst):
     return _checkers.get(inst.__class__, _defaultChecker)
 
+
 def moduleChecker(module):
     """
     Return the :class:`zope.security.interfaces.IChecker` defined for the
@@ -713,11 +745,13 @@ _setChecker = NamesChecker([
     '__le__', '__ge__',
 ])
 
+
 class _BasicTypes(dict):
     """Basic Types Dictionary
 
     Make sure that checkers are really updated, when a new type is added.
     """
+
     def __setitem__(self, name, value):
         dict.__setitem__(self, name, value)
         _checkers[name] = value
@@ -734,6 +768,7 @@ class _BasicTypes(dict):
         dict.update(self, d)
         _checkers.update(d)
 
+
 _basic_types = {
     object: NoProxy,
     int: NoProxy,
@@ -742,7 +777,7 @@ _basic_types = {
     type(None): NoProxy,
     str: NoProxy,
     bytes: NoProxy,
-    Message: NoProxy, # Messages are immutable, so it's okay
+    Message: NoProxy,  # Messages are immutable, so it's okay
     bool: NoProxy,
     datetime.timedelta: NoProxy,
     datetime.datetime: NoProxy,
@@ -750,10 +785,10 @@ _basic_types = {
     datetime.time: NoProxy,
     datetime.tzinfo: NoProxy,
 }
-if PYTHON2:
-    _basic_types[long] = NoProxy
-    _basic_types[unicode] = NoProxy
-else: # pragma: no cover
+if PYTHON2:  # pragma: no cover
+    _basic_types[long] = NoProxy  # noqa: F821 undefined name 'long'
+    _basic_types[unicode] = NoProxy  # noqa: F821 undefined name 'unicode'
+else:
     _basic_types[type({}.values())] = NoProxy
     _basic_types[type({}.keys())] = NoProxy
     _basic_types[type({}.items())] = NoProxy
@@ -784,26 +819,29 @@ BasicTypes_examples = {
     Message: Message('message', domain='hello')
 }
 
-if PYTHON2:
-    BasicTypes_examples[unicode] = u'uabc'
-    BasicTypes_examples[long] = long(65536)
+if PYTHON2:  # pragma: no cover
+    BasicTypes_examples[unicode] = u'uabc'  # noqa: F821 undefined name
+    BasicTypes_examples[long] = long(65536)  # noqa: F821 undefined name
 
 
 class _Sequence(object):
     def __len__(self):
         raise NotImplementedError()
+
     def __getitem__(self, i):
         raise NotImplementedError()
 
+
 _Declaration_checker = InterfaceChecker(
     IDeclaration,
     _implied=CheckerPublic,
     subscribe=CheckerPublic,
     unsubscribe=CheckerPublic,
     __call__=CheckerPublic,
-    )
+)
 
-def f(): # pragma: no cover
+
+def f():  # pragma: no cover
     yield f
 
 
@@ -837,7 +875,7 @@ _default_checkers = {
                                    'max', 'min', 'normalize',
                                    'quantize', 'remainder_near',
                                    'same_quantum', 'sqrt',
-                                   'to_eng_string', 'to_integral' ]),
+                                   'to_eng_string', 'to_integral']),
 
     # YAGNI: () a rock
     tuple: NamesChecker(['__getitem__', '__getslice__', '__add__', '__radd__',
@@ -849,10 +887,12 @@ _default_checkers = {
     types.MethodType: _callableChecker,
     types.BuiltinFunctionType: _callableChecker,
     types.BuiltinMethodType: _callableChecker,
+    # method-wrapper
+    type(().__repr__): _callableChecker,
     type: _typeChecker,
     types.ModuleType: lambda module: _checkers.get(module, _namedChecker),
-    type(iter([])): _iteratorChecker, # Same types in Python 2.2.1,
-    type(iter(())): _iteratorChecker, # different in Python 2.3.
+    type(iter([])): _iteratorChecker,  # Same types in Python 2.2.1,
+    type(iter(())): _iteratorChecker,  # different in Python 2.3.
     type(iter({})): _iteratorChecker,
     type(iter(set())): _iteratorChecker,
     type(iter(_Sequence())): _iteratorChecker,
@@ -875,15 +915,15 @@ _default_checkers = {
     zope.interface.declarations.Declaration: _Declaration_checker,
     abc.ABCMeta: _typeChecker,
 }
-if PYTHON2:
+if PYTHON2:  # pragma: no cover
     _default_checkers[types.ClassType] = _typeChecker
     _default_checkers[types.InstanceType] = _instanceChecker
     # slot description
-    _default_checkers[type(().__getslice__)] = _callableChecker
     _default_checkers[type({}.iteritems())] = _iteratorChecker
     _default_checkers[type({}.iterkeys())] = _iteratorChecker
     _default_checkers[type({}.itervalues())] = _iteratorChecker
 
+
 def _fixup_dictlike(dict_type):
     empty_dict = dict_type()
     populated_dict = dict_type({1: 2})
@@ -898,20 +938,25 @@ def _fixup_dictlike(dict_type):
             if iter_type not in _default_checkers:
                 _default_checkers[iter_type] = _iteratorChecker
 
+
 def _fixup_odict():
     # OrderedDicts have three different implementations: Python 2 (pure
     # python, returns generators and lists), Python <=3.4 (pure Python,
     # uses view classes) and CPython 3.5+ (implemented in C). These should
     # all be iterable.
     from collections import OrderedDict
-    _fixup_dictlike(OrderedDict)
+
+    # The `_fixup_dictlike` is detected as undefined because it is deleted
+    # later on but this function is called beforehand:
+    _fixup_dictlike(OrderedDict)  # noqa: F821 undefined name '_fixup_dictlike'
+
 
 _fixup_odict()
 del _fixup_odict
 
 try:
-    import BTrees
-except ImportError: # pragma: no cover
+    import BTrees  # noqa: F401 'BTrees' imported but unused
+except ImportError:  # pragma: no cover
     pass
 else:
     # The C implementation of BTree.items() is its own iterator
@@ -934,13 +979,16 @@ else:
             for family_name in ('family32', 'family64'):
                 family = getattr(BTrees, family_name)
                 btree = getattr(family, name).BTree
-                _fixup_dictlike(btree)
+                # The `_fixup_dictlike` is detected as undefined because it is
+                # deleted later on but this function is called beforehand:
+                _fixup_dictlike(btree)  # noqa: F821 undefined name
 
     _fixup_btrees()
     del _fixup_btrees
 
 del _fixup_dictlike
 
+
 def _fixup_zope_interface():
     # Make sure the provided and implementedBy objects
     # can be iterated.
@@ -948,9 +996,10 @@ def _fixup_zope_interface():
     # we use NoProxy to be sure that the results (of iteration or not) are not
     # proxied. On Python 2, these objects are builtin and don't go through the
     # checking process at all, much like BTrees, so NoProxy is necessary for
-    # compatibility. On Python 3, prior to this, iteration was simply not allowed.
-    from zope.interface import providedBy
+    # compatibility. On Python 3, prior to this, iteration was simply not
+    # allowed.
     from zope.interface import alsoProvides
+    from zope.interface import providedBy
 
     class I1(Interface):
         pass
@@ -959,11 +1008,10 @@ def _fixup_zope_interface():
         pass
 
     @implementer(I1)
-    class O(object):
+    class Obj(object):
         pass
 
-    o = O()
-
+    o = Obj()
 
     # This will be athe zope.interface.implementedBy from the class
     # a zope.interface.declarations.Implements
@@ -973,6 +1021,7 @@ def _fixup_zope_interface():
     # This will be the zope.interface.Provides from the instance
     _default_checkers[type(providedBy(o))] = NoProxy
 
+
 _fixup_zope_interface()
 del _fixup_zope_interface
 
@@ -998,8 +1047,10 @@ def _fixup_itertools():
         _default_checkers[type_grouper] = _iteratorChecker
 
     # There are also many other custom types in itertools that need the
-    # same treatment. See a similar list in test_checker.py:test_itertools_checkers
-    pred = lambda x: x
+    # same treatment. See a similar list in
+    # test_checker.py:test_itertools_checkers
+    def pred(x):
+        return x
     iterable = (1, 2, 3)
     pred_iterable = (pred, iterable)
     missing_in_py3 = {'ifilter', 'ifilterfalse', 'imap',
@@ -1034,7 +1085,8 @@ def _fixup_itertools():
             func = getattr(itertools, func)
         except AttributeError:
             assert func in missing, "Expected %s but not found" % (func,)
-            continue # pragma: no cover This is hit on Python 2, but it doesn't always show
+            # The following line is hit on PY2, but it doesn't always show:
+            continue  # pragma: no cover
 
         result = func(*args)
         if func == itertools.tee:
@@ -1043,19 +1095,22 @@ def _fixup_itertools():
         if tresult not in _default_checkers:
             _default_checkers[tresult] = _iteratorChecker
 
+
 _fixup_itertools()
 del _fixup_itertools
 
+
 def _clear():
     _checkers.clear()
     _checkers.update(_default_checkers)
     _checkers.update(BasicTypes)
 
+
 _clear()
 
 try:
     from zope.testing.cleanup import addCleanUp
-except ImportError: # pragma: no cover
+except ImportError:  # pragma: no cover
     pass
 else:
     addCleanUp(_clear)
